@@ -10,15 +10,11 @@
 #   Andy Wardley   <abw@wardley.org>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-2006 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-# REVISION
-#   $Id: Stash.pm,v 2.105 2006/05/30 17:01:29 abw Exp $
-#
 #============================================================================
 
 package Template::Stash;
@@ -26,10 +22,18 @@ package Template::Stash;
 use strict;
 use warnings;
 use Template::VMethods;
+use Template::Exception;
+use Scalar::Util qw( blessed reftype );
 
-our $VERSION = 2.90;
-our $DEBUG   = 0 unless defined $DEBUG;
-our $PRIVATE = qr/^[_.]/;
+our $VERSION    = 2.91;
+our $DEBUG      = 0 unless defined $DEBUG;
+our $PRIVATE    = qr/^[_.]/;
+our $UNDEF_TYPE = 'var.undef';
+our $UNDEF_INFO = 'undefined variable: %s';
+
+# alias _dotop() to dotop() so that we have a consistent method name
+# between the Perl and XS stash implementations
+*dotop = \&_dotop;
 
 
 #------------------------------------------------------------------------
@@ -147,7 +151,7 @@ sub clone {
 
     # look out for magical 'import' argument which imports another hash
     my $import = $params->{ import };
-    if (defined $import && UNIVERSAL::isa($import, 'HASH')) {
+    if (defined $import && ref $import eq 'HASH') {
         delete $params->{ import };
     }
     else {
@@ -224,7 +228,9 @@ sub get {
         $result = $self->_dotop($root, $ident, $args);
     }
 
-    return defined $result ? $result : $self->undefined($ident, $args);
+    return defined $result 
+        ? $result 
+        : $self->undefined($ident, $args);
 }
 
 
@@ -284,7 +290,6 @@ sub set {
 # 
 # Returns a "reference" to a particular item.  This is represented as a 
 # closure which will return the actual stash item when called.  
-# WARNING: still experimental!
 #------------------------------------------------------------------------
 
 sub getref {
@@ -331,7 +336,7 @@ sub update {
 
     # look out for magical 'import' argument to import another hash
     my $import = $params->{ import };
-    if (defined $import && UNIVERSAL::isa($import, 'HASH')) {
+    if (defined $import && ref $import eq 'HASH') {
         @$self{ keys %$import } = values %$import;
         delete $params->{ import };
     }
@@ -348,8 +353,39 @@ sub update {
 #------------------------------------------------------------------------
 
 sub undefined {
-    my ($self, $ident, $args);
-    return '';
+    my ($self, $ident, $args) = @_;
+
+    if ($self->{ _STRICT }) {
+        # Sorry, but we can't provide a sensible source file and line without
+        # re-designing the whole architecture of TT (see TT3)
+        die Template::Exception->new(
+            $UNDEF_TYPE, 
+            sprintf(
+                $UNDEF_INFO, 
+                $self->_reconstruct_ident($ident)
+            )
+        ) if $self->{ _STRICT };
+    }
+    else {
+        # There was a time when I thought this was a good idea. But it's not.
+        return '';
+    }
+}
+
+sub _reconstruct_ident {
+    my ($self, $ident) = @_;
+    my ($name, $args, @output);
+    my @input = ref $ident eq 'ARRAY' ? @$ident : ($ident);
+
+    while (@input) {
+        $name = shift @input;
+        $args = shift @input || 0;
+        $name .= '(' . join(', ', map { /^\d+$/ ? $_ : "'$_'" } @$args) . ')'
+            if $args && ref $args eq 'ARRAY';
+        push(@output, $name);
+    }
+    
+    return join('.', @output);
 }
 
 
@@ -383,7 +419,7 @@ sub undefined {
 sub _dotop {
     my ($self, $root, $item, $args, $lvalue) = @_;
     my $rootref = ref $root;
-    my $atroot  = ($root eq $self);
+    my $atroot  = (blessed $root && $root->isa(ref $self));
     my ($value, @result);
 
     $args ||= [ ];
@@ -444,7 +480,9 @@ sub _dotop {
     # doesn't appear to work with CGI, returning true for the first call
     # and false for all subsequent calls. 
     
-    elsif (ref($root) && UNIVERSAL::can($root, 'can')) {
+    # UPDATE: that doesn't appear to be the case any more
+    
+    elsif (blessed($root) && $root->can('can')) {
 
         # if $root is a blessed reference (i.e. inherits from the 
         # UNIVERSAL object base class) then we call the item as a method.
@@ -453,7 +491,7 @@ sub _dotop {
         eval { @result = $root->$item(@$args); };       
         
         if ($@) {
-            # temporary hack - required to propogate errors thrown
+            # temporary hack - required to propagate errors thrown
             # by views; if $@ is a ref (e.g. Template::Exception
             # object then we assume it's a real error that needs
             # real throwing
@@ -462,7 +500,7 @@ sub _dotop {
             die $@ if ref($@) || ($@ !~ /Can't locate object method "\Q$item\E" via package "\Q$class\E"/);
 
             # failed to call object method, so try some fallbacks
-            if (UNIVERSAL::isa($root, 'HASH') ) {
+            if (reftype $root eq 'HASH') {
                 if( defined($value = $root->{ $item })) {
                     return $value unless ref $value eq 'CODE';      ## RETURN
                     @result = &$value(@$args);
@@ -470,8 +508,11 @@ sub _dotop {
                 elsif ($value = $HASH_OPS->{ $item }) {
                     @result = &$value($root, @$args);
                 }
+                elsif ($value = $LIST_OPS->{ $item }) {
+                    @result = &$value([$root], @$args);
+                }
             }
-            elsif (UNIVERSAL::isa($root, 'ARRAY') ) {
+            elsif (reftype $root eq 'ARRAY') {
                 if( $value = $LIST_OPS->{ $item }) {
                    @result = &$value($root, @$args);
                 }
@@ -565,7 +606,7 @@ sub _assign {
         return ($root->[$item] = $value)            ## RETURN
             unless $default && $root->{ $item };
     }
-    elsif (UNIVERSAL::isa($root, 'UNIVERSAL')) {
+    elsif (blessed($root)) {
         # try to call the item as a method of an object
         
         return $root->$item(@$args, $value)         ## RETURN
@@ -644,18 +685,6 @@ sub _dump_frame {
 
 __END__
 
-
-#------------------------------------------------------------------------
-# IMPORTANT NOTE
-#   This documentation is generated automatically from source
-#   templates.  Any changes you make here may be lost.
-# 
-#   The 'docsrc' documentation source bundle is available for download
-#   from http://www.template-toolkit.org/docs.html and contains all
-#   the source templates, XML files, scripts, etc., from which the
-#   documentation for the Template Toolkit is built.
-#------------------------------------------------------------------------
-
 =head1 NAME
 
 Template::Stash - Magical storage for template variables
@@ -663,41 +692,41 @@ Template::Stash - Magical storage for template variables
 =head1 SYNOPSIS
 
     use Template::Stash;
-
+    
     my $stash = Template::Stash->new(\%vars);
-
+    
     # get variable values
     $value = $stash->get($variable);
     $value = $stash->get(\@compound);
-
+    
     # set variable value
     $stash->set($variable, $value);
     $stash->set(\@compound, $value);
-
+    
     # default variable value
     $stash->set($variable, $value, 1);
     $stash->set(\@compound, $value, 1);
-
+    
     # set variable values en masse
     $stash->update(\%new_vars)
-
+    
     # methods for (de-)localising variables
     $stash = $stash->clone(\%new_vars);
     $stash = $stash->declone();
 
 =head1 DESCRIPTION
 
-The Template::Stash module defines an object class which is used to store
+The C<Template::Stash> module defines an object class which is used to store
 variable values for the runtime use of the template processor.  Variable
 values are stored internally in a hash reference (which itself is blessed 
-to create the object) and are accessible via the get() and set() methods.
+to create the object) and are accessible via the L<get()> and L<set()> methods.
 
 Variables may reference hash arrays, lists, subroutines and objects
 as well as simple values.  The stash automatically performs the right
 magic when dealing with variables, calling code or object methods,
 indexing into lists, hashes, etc.
 
-The stash has clone() and declone() methods which are used by the
+The stash has L<clone()> and L<declone()> methods which are used by the
 template processor to make temporary copies of the stash for
 localising changes made to variables.
 
@@ -705,8 +734,8 @@ localising changes made to variables.
 
 =head2 new(\%params)
 
-The new() constructor method creates and returns a reference to a new
-Template::Stash object.  
+The C<new()> constructor method creates and returns a reference to a new
+C<Template::Stash> object.  
 
     my $stash = Template::Stash->new();
 
@@ -714,11 +743,11 @@ A hash reference may be passed to provide variables and values which
 should be used to initialise the stash.
 
     my $stash = Template::Stash->new({ var1 => 'value1', 
-				       var2 => 'value2' });
+                                       var2 => 'value2' });
 
 =head2 get($variable)
 
-The get() method retrieves the variable named by the first parameter.
+The C<get()> method retrieves the variable named by the first parameter.
 
     $value = $stash->get('var1');
 
@@ -726,15 +755,15 @@ Dotted compound variables can be retrieved by specifying the variable
 elements by reference to a list.  Each node in the variable occupies
 two entries in the list.  The first gives the name of the variable
 element, the second is a reference to a list of arguments for that 
-element, or 0 if none.
+element, or C<0> if none.
 
     [% foo.bar(10).baz(20) %]
-
+    
     $stash->get([ 'foo', 0, 'bar', [ 10 ], 'baz', [ 20 ] ]);
 
 =head2 set($variable, $value, $default)
 
-The set() method sets the variable name in the first parameter to the 
+The C<set()> method sets the variable name in the first parameter to the 
 value specified in the second.
 
     $stash->set('var1', 'value1');
@@ -744,70 +773,111 @@ set only if it did not have a true value before.
 
     $stash->set('var2', 'default_value', 1);
 
-Dotted compound variables may be specified as per get() above.
+Dotted compound variables may be specified as per L<get()> above.
 
     [% foo.bar = 30 %]
-
+    
     $stash->set([ 'foo', 0, 'bar', 0 ], 30);
 
-The magical variable 'IMPORT' can be specified whose corresponding
+The magical variable 'C<IMPORT>' can be specified whose corresponding
 value should be a hash reference.  The contents of the hash array are
 copied (i.e. imported) into the current namespace.
 
     # foo.bar = baz, foo.wiz = waz
     $stash->set('foo', { 'bar' => 'baz', 'wiz' => 'waz' });
-
+    
     # import 'foo' into main namespace: bar = baz, wiz = waz
     $stash->set('IMPORT', $stash->get('foo'));
 
+=head2 update($variables)
+
+This method can be used to set or update several variables in one go.
+
+    $stash->update({
+        foo => 10,
+        bar => 20,
+    });
+
+=head2 getref($variable)
+
+This undocumented feature returns a closure which can be called to get the
+value of a variable.  It is used to implement variable references which are
+evaluated lazily.
+
+    [% x = \foo.bar.baz %]          # x is a reference to foo.bar.baz
+    [% x %]                         # evalautes foo.bar.baz
+
 =head2 clone(\%params)
 
-The clone() method creates and returns a new Template::Stash object which
-represents a localised copy of the parent stash.  Variables can be
-freely updated in the cloned stash and when declone() is called, the
-original stash is returned with all its members intact and in the
-same state as they were before clone() was called.
+The C<clone()> method creates and returns a new C<Template::Stash> object
+which represents a localised copy of the parent stash. Variables can be freely
+updated in the cloned stash and when L<declone()> is called, the original stash
+is returned with all its members intact and in the same state as they were
+before C<clone()> was called.
 
-For convenience, a hash of parameters may be passed into clone() which 
+For convenience, a hash of parameters may be passed into C<clone()> which 
 is used to update any simple variable (i.e. those that don't contain any 
-namespace elements like 'foo' and 'bar' but not 'foo.bar') variables while 
-cloning the stash.  For adding and updating complex variables, the set() 
-method should be used after calling clone().  This will correctly resolve
+namespace elements like C<foo> and C<bar> but not C<foo.bar>) variables while 
+cloning the stash.  For adding and updating complex variables, the L<set()> 
+method should be used after calling C<clone().>  This will correctly resolve
 and/or create any necessary namespace hashes.
 
 A cloned stash maintains a reference to the stash that it was copied 
-from in its '_PARENT' member.
+from in its C<_PARENT> member.
 
 =head2 declone()
 
-The declone() method returns the '_PARENT' reference and can be used to
+The C<declone()> method returns the C<_PARENT> reference and can be used to
 restore the state of a stash as described above.
 
+=head2 define_vmethod($type, $name, $code)
+
+This method can be used to define new virtual methods.  The first argument
+should be either C<scalar> or C<item> to define scalar virtual method, C<hash>
+to define hash virtual methods, or either C<array> or C<list> for list virtual
+methods.  The second argument should be the name of the new method.  The third
+argument should be a reference to a subroutine implementing the method.  The
+data item on which the virtual method is called is passed to the subroutine as
+the first argument.
+
+    $stash->define_vmethod(
+        item => ucfirst => sub {
+            my $text = shift;
+            return ucfirst $text
+        }
+    );
+
+=head1 INTERNAL METHODS
+
+=head2 dotop($root, $item, \@args, $lvalue)
+
+This is the core C<dot> operation method which evaluates elements of 
+variables against their root.
+
+=head2 undefined($ident, $args)
+
+This method is called when L<get()> encounters an undefined value.  If the 
+C<STRICT|Template::Manual::Config#STRICT> option is in effect then it will
+throw an exception indicating the use of an undefined value.  Otherwise it
+will silently return an empty string.
+
+The method can be redefined in a subclass to implement alternate handling
+of undefined values.
+
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@wardley.orgE<gt>
-
-L<http://wardley.org/|http://wardley.org/>
-
-
-
-
-=head1 VERSION
-
-2.9, distributed as part of the
-Template Toolkit version 2.18, released on 09 February 2007.
+Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
-
+Copyright (C) 1996-2012 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template|Template>, L<Template::Context|Template::Context>
+L<Template>, L<Template::Context>
 
 =cut
 
